/*
 * Main.java
 */

package wsspsample;

import com.sun.net.httpserver.Authenticator;
import javax.xml.ws.*;
import javax.xml.bind.*;
import javax.xml.*;
import javax.xml.transform.stream.*;
import java.io.*;
import java.util.*;

import wsspsample.webref.search.*;
import wsspsample.xom.query.*;
import wsspsample.xom.response.*;

/**
 *
 * @author Administrator
 */
public class Main {
    
    /** Creates a new instance of Main */
    public Main() {
                //String sResponse = qs.getQueryServiceSoap().query("<QueryPacket xmlns=\"urn:Microsoft.Search.Query\"><Query><Context><QueryText type=\"STRING\" language=\"en-us\">sharepoint</QueryText></Context></Query></QueryPacket>");
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws JAXBException {
        // TODO code application logic here
        //System.out.println("Welcome to Java SharePoint Search Client Sample\n");
        
        QueryService qs = new QueryService();
        QueryServiceSoap qsp = qs.getQueryServiceSoap(); //qs.getQueryServiceSoap does not seem to return the same reference, so we need to hang on to this
        BindingProvider bp = (BindingProvider)qsp;
        
        // If credentials are required we set them here
        //bp.getRequestContext().put(BindingProvider.USERNAME_PROPERTY, "Administrator");
        //bp.getRequestContext().put(BindingProvider.PASSWORD_PROPERTY, "pass@word1");
        bp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "http://barbie/_vti_bin/spsearch.asmx");

        // Check the server status
        String status = qsp.status();
        System.out.println("Server Status: " + status + "\n");

        // Create a Search Query Packet Object
        QueryPacket qp = new QueryPacket();
        QueryType qt = new QueryType();
        ContextType ct = new ContextType();
        QueryTextType ctt = new QueryTextType();
        
        ct.setQueryText(ctt);
        qt.setContext(ct);
        qp.setQuery(qt);
        
        // Set search values
        ctt.setValue("sharepoint");
        ctt.setType("STRING");      //This is default - not necessary
        ctt.setLanguage("en-us");   //This is default - not necessary
        
        // Call the web service query
        String sResponse = qsp.query(SerializeQuery(qp));

        // Deserialize the response
        ResponsePacket resp = DeserializeResponse(sResponse);
        
        // Print the result out to the standard output stream
        PrintResults(resp.getResponse());
    }
    
    // Take a QueryPacket and serialize it to XML to pass in to the Search::Query method
    public static String SerializeQuery(QueryPacket qp)
    {
        try
        {
            JAXBContext jc = JAXBContext.newInstance("wsspsample.xom.query");

            Marshaller ma = jc.createMarshaller();

            ma.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            ma.marshal(qp, os);
            return os.toString();
        }
        catch(JAXBException ex)
        {
            return "";
        }
    }
    
    public static ResponsePacket DeserializeResponse(String s)
    {
        try
        {
            JAXBContext jc = JAXBContext.newInstance("wsspsample.xom.response");

            Unmarshaller um = jc.createUnmarshaller();
            StreamSource source = new StreamSource(new StringReader(s));

            return (ResponsePacket)um.unmarshal(source);
        }
        catch (JAXBException ex)
        {
            return new ResponsePacket();
        }
    }
    
    // Print ResponsePacket.Response to Console
    public static void PrintResults(ResponsePacket.Response resp)
    {
        // Print results to console
        // If we are successful we can iterate over the results, zero results will return a failure code
        if ("SUCCESS".compareTo(resp.getStatus()) == 0)
        {
            System.out.println("================== SEARCH RESULTS =====================");
            System.out.println(resp.getRange().getCount() + " Results Found");
            
            List<Document> docs = resp.getRange().getResults().getDocument();

            String title = "";
            String desc = "";
            String url = "";

            for (int i = 0; i < docs.size(); i++)
            {
                title = docs.get(i).getTitle();
                desc = docs.get(i).getDescription();
                url = docs.get(i).getAction().getLinkUrl().getValue();
                System.out.println(title);
                if (desc.length() > 0)
                    System.out.println(desc);
                System.out.println(url);
                System.out.println("-------------------------------------------------------");
            }
        } else {
            System.out.println("Response Status: " + resp.getStatus());
        }
    }
}
